"""
Licensed Materials - Property of IBM

5724-X89 Copyright IBM Corp. 2009 All Rights Reserved.

US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
"""

import os
import re
import sys
import constant

__all__ = ['pathsep', 'allow_loop_device']

# The pathsep attribute is equivalent to os.path.sep in Python versions greater than 2.2
pathsep = '/'
if hasattr(sys, 'haswindowsversion'):
    pathsep = '\\'

allow_loop_device = False # For testing. When True, allows locating files on loop devices

def _regexize(s):
    """Change a common string to python-compatible regular expression."""
    ret = s.replace('.', '\\.')
    ret = ret.replace('*', '.*')
    return '^' + ret + '$'

def abs_join(p, *a):
    """ """
    return os.path.abspath(os.path.join(p, *a))

def writefile(filename, content, append=False):
    """OS-independent write file API."""
    mkdirs(filename)
    if jython():
        # In Jython environment
        if append:
            if os.path.exists(filename): os.remove(filename)
        from java.io import FileWriter
        fw = FileWriter(os.File(filename))
        fw.write(content)
        fw.close()
    else:
        f = None
        if append:
            f = open(filename, 'a')
        else:
            f = open(filename, 'w')
        f.write(content)
        f.close()

def mkdirs(filename):
    pathname = os.path.dirname(filename)
    if os.path.exists(pathname) == False:
        # Make directories
        os.makedirs(pathname)    

def _mktmpdir():
    # Make a temporary directory
    import tempfile
    tmpdir = tempfile.mktemp()
    os.makedirs(tmpdir)

    return tmpdir
    
def get_exe():
    if op_sys() == constant.OS_WIN:
        exe = 'AE.bat'
    else:
        exe = 'AE.sh'
    return exe

def lock(key=None):
    """Try to get the runtime lock. Wait until success."""
    return
    import time
    if None == key:
        key = str(os.getpid())
    if not os.path.exists(constant.LOCK_DIR):
        print "[%s:%s@%s] create lock dir" % (str(os.getpid()) , key, str(time.time()))
        try:
            os.mkdir(constant.LOCK_DIR)
        except OSError, e:
            import errno
            if e.errno != errno.EEXIST:
                raise
    dirlist = os.listdir(constant.LOCK_DIR)
    print "[%s:%s@%s] list dir: %s" % (str(os.getpid()) , key, str(time.time()), str(dirlist))
    while len(dirlist) > 0:
        time.sleep(1)
        dirlist = os.listdir(constant.LOCK_DIR)
        print "[%s:%s@%s] waited for 1s, list dir again: %s" % (str(os.getpid()) , key, str(time.time()), str(dirlist))
    print "[%s:%s@%s] no pre-lock or lock, write pre file" % (str(os.getpid()) , key, str(time.time()))
    writefile(abs_join(constant.LOCK_DIR, key+'.pre'), "")
    time.sleep(0.1)
    dirlist = os.listdir(constant.LOCK_DIR)
    print "[%s:%s@%s] list dir: %s" % (str(os.getpid()) , key, str(time.time()), str(dirlist))
    locks = [ item for item in dirlist if item.endswith('.lock') ]
    pres = [ item for item in dirlist if item.endswith('.pre') ]
    while len(locks) > 0:
        time.sleep(1)
        dirlist = os.listdir(constant.LOCK_DIR)
        print "[%s:%s@%s] waited for 1s, list dir again: %s" % (str(os.getpid()) , key, str(time.time()), str(dirlist))
        locks = [ item for item in dirlist if item.endswith('.lock') ]
        pres = [ item for item in dirlist if item.endswith('.pre') ]
    pres.sort()
    print "[%s:%s@%s] no lock, sort pre files: %s" % (str(os.getpid()) , key, str(time.time()), str(pres))
    while pres[0] != key+'.pre':
        time.sleep(1)
        dirlist = os.listdir(constant.LOCK_DIR)
        pres = [ item for item in dirlist if item.endswith('.pre') ]
        pres.sort()
        print "[%s:%s@%s] I'm not the first place in pres, waited and get new list: %s" % (str(os.getpid()) , key, str(time.time()), str(pres))
    writefile(abs_join(constant.LOCK_DIR, key+'.lock'), "")
    print "[%s:%s@%s] after get lock" % (str(os.getpid()) , key, str(time.time()))
    

def unlock(key=None):
    """Release the runtime lock."""
    return
    if None == key:
        key = str(os.getpid())
    try:
        os.remove(abs_join(constant.LOCK_DIR, key+'.lock'))
        os.remove(abs_join(constant.LOCK_DIR, key+'.pre'))
    except:
        pass
        

def jython():
    """Test whether the runtime is Jython. Return True if in Jython runtime"""
    if os.name == 'java': return True
    return False
    
def op_sys():
    """Return OS name
    Windows - Windows
    Linux - Most Linux distributions
    Aix - IBM AIX"""
    if jython() == True:
        from java.lang import System
        osname = System.getProperty("os.name")
        if osname == 'Linux': return constant.OS_LINUX
        elif osname == 'AIX': return constant.OS_AIX
        elif osname == 'Windows': return constant.OS_WIN
    elif jython() == False:
        if os.name == 'nt': return constant.OS_WIN
        if os.name == 'posix':
            if sys.platform[:5] == "linux": return constant.OS_LINUX
            elif sys.platform[:3] == "aix": return constant.OS_AIX

def locatefile(dir_name, patterns):
    """Return the filename match the pattern in specific directory"""
    if dir_name == None:
        return __locatefile(patterns)
    else:
        ret = []
        for pattern in patterns:
            if pathsep in pattern:
                dir_name = abs_join(dir_name, os.path.dirname(pattern))
                pattern = os.path.basename(pattern)
            if os.path.exists(dir_name) == False or os.path.isdir(dir_name) == False:
                pass
            else:
                files = os.listdir(dir_name)
                for f in files:
                    if re.match(_regexize(pattern), \
                        os.path.basename(f)):
                        ret.append(abs_join(dir_name, f))
        return ret


def __locatefile(patterns):
    """Return the filename match the pattern in default search directories"""
    name = op_sys()
    dirs = __listdirs()

    if re.match('^Windows', name):
        for d in dirs:
            files =  locatefile(d, patterns)
            if len(files) > 0: return files
    else:
        for d in dirs:
            if os.path.exists(d):
                if d.startswith('/dev/'):
                    # Search file in device
                    filename = __searchdev(d, patterns)
                    if len(filename) > 0: return filename
                else:
                    # Search file in common directory
                    filename = locatefile(d, patterns)
                    if len(filename) > 0: return filename
    return []

def __searchdev(dev, patterns):
    """Search the file whose name match the pattern in specific device"""
    # Check whether this device is mounted already
    mount = os.popen('mount | grep %s | cut -d " " -f 3' % dev).readline()[:-1]
    mp = constant.DEFAULT_MOUNT_POINT
    ret = []
    
    if not mount:
        # If not mounted, try to mount and search the file
        # Check mount point
        if not os.path.exists(mp):
            os.makedirs(mp)
        # Mount this device
        retcode = os.system('mount ' + dev + ' ' + mp)
        if retcode != 0:
            return ret

    # Search the file
    files = []
    if mount:
        files = locatefile(mount, patterns)
    else:
        files = locatefile(mp, patterns)

    if len(files) > 0:
        # Get a recorded temporary directory
        tmpdir = _mktmpdir()

        # Copy the file from device to the temporary directory
        import shutil
        for f in files:
            shutil.copy(f, tmpdir)
            ret.append(abs_join(tmpdir, os.path.basename(f)))

    if not mount:
        # Fail to search, unmount this device, and return None
        os.system('umount ' + mp)
    return ret

def __listdirs():
    """Return a list for search directories"""
    name = op_sys()
    if name == constant.OS_WIN:
        drives = 'cdefghijklmnopqrstuvwxyz'
        roots = []
        for drive in drives:
            root = drive + ':\\'
            if os.path.exists(root): roots.append(root)
        return roots
    elif name == constant.OS_LINUX:
        # list all devices in linux
        ret = []
        block = '/sys/block'
        for b in os.listdir(block):
            if (re.match("^loop", b) and not allow_loop_device) \
                or re.match("^ram", b) \
                or re.match("^md", b):
                pass
            else:
                flag = 0
                for s in os.listdir(abs_join(block, b)):
                    if re.match("^" + b, s):
                        flag = 1
                        ret.append('/dev/' + s)
                if flag == 0: ret.append('/dev/' + b)
	ret.append('/etc/ovf-transport/')
        return ret
    elif name == constant.OS_AIX:
        # for Aix, search /etc/ovf-transport
        ret = []
        ret.append('/etc/ovf-transport/')
        return ret

def escape(s):
    """Escape meta-characters in string """
    ret = s.replace("\n", ' ')
    ret = ret.replace("\r", ' ')
    ret = ret.replace('"', '\\"')
    return ret

def spawnvp(filename, args):
    if jython() == True:
        from java.lang import ProcessBuilder
        cmdarray = [filename]
        cmdarray.extend(args)
        pb = ProcessBuilder(cmdarray)
        return pb.start()
    elif op_sys() == constant.OS_WIN:
        for path in os.environ['PATH'].split(';'):
            if os.path.exists(abs_join(path, filename)):
                return os.spawnve(os.P_NOWAIT, abs_join(path, filename), args, os.environ)
    else:
        return os.spawnvpe(os.P_NOWAIT, filename, args, os.environ)
    
def deltree(dirname):
    if os.path.exists(dirname):
        if os.path.isdir(dirname):
            for f in os.listdir(dirname):
                if os.path.isdir(abs_join(dirname, f)): deltree(abs_join(dirname, f))
                else: os.remove(abs_join(dirname, f))
            os.rmdir(dirname)
        else:
            os.remove(dirname)

if __name__ == '__main__':
    __searchdev('/dev/sda1', '')
