# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#


DB2_SOURCE_CMD="source /home/$DB2INST/sqllib/db2profile"

is_db2_started () {	
#   local result=`ps auxww | grep db2sysc | grep $DB2INST | awk '{print $1}'`    
    
    $DB2_MONITOR_SCRIPT $DB2INST
    result=$?
    if [ "$result" -eq $TSA_ONLINE -o "$result" -eq $TSA_PENDING_ONLINE ]; then
        debug "is_db2_started: 1"
        return 1
    else
        debug "is_db2_started: 0"
        return 0
    fi	
}

start_db2 () {
	
	debug "Start DB2"
	
	is_db2_started
	if [ $? -eq 0 ]; then 
		
        local result=`LC_ALL=C su - $DB2INST -c "$DB2_SOURCE_CMD; db2start" 2>&1`
		debug "$result"
		    
	    for i in {1..160}
	    do
		    is_db2_started
		    if [ $? -eq 1 ]; then 
	        	return 0
	        else
	            sleep 3
	        fi
	    done
	    return 1
    fi
    
    return 0
}

stop_db2_internal () {

#    local result=`su - $DB2INST -c "$DB2_SOURCE_CMD; db2stop" 2>&1`
#    debug "stop_db2_internal:$result"
    debug "Stopping db2"
    $DB2_STOP_SCRIPT $DB2INST
}

#stop_db2_force_internal () {
#
#    local result=`su - $DB2INST -c "$DB2_SOURCE_CMD; db2stop force" 2>&1`
#    debug "stop_db2_force_internal:$result"
#}

            
stop_db2 () {
    
    debug "Stop DB2"
    local rmdr

    is_db2_started
    if [ $? -eq 1 ]; then 
        
        stop_db2_internal
        		    
        for i in {1..160}
        do
            is_db2_started
            if [ $? -eq 0 ]; then 
                return 0
            else
                # Try stopping it again every so often
                let "rmdr = i % 5"
                if [ $rmdr -eq 0 ]; then
                    stop_db2_internal
                else
                    sleep 5
                fi
            fi
        done
        # We couldn't stop it normally, try force once
#        stop_db2_force_internal
        is_db2_started
        if [ $? -eq 0 ]; then 
            return 0
        fi
        
        return 1
    fi
    
    return 0
}


dlt_db2_rsrc () {
    debug "Delete DB2 resources"
	take_rg_offline_n_wait_nonfatal $DB2_RG
    rmrsrc -s "Name=='$DB2_DB_RSRC_NAME'" IBM.Application Force=1
    rmrg -s "Name=='$DB2_RG'"
}

crt_db2_rsrc () {
	
    # Only create the resource if it doesn't already exist
    lsrg -A d -g $DB2_DB_RSRC_NAME &>/dev/null
    if [ $? -eq 6 ]; then
        debug "Create the DB2 resources"
        mkrsrc IBM.Application \
        Name="$DB2_DB_RSRC_NAME" \
        StartCommand="$DB2_START_SCRIPT $DB2INST" \
        StopCommand="$DB2_STOP_SCRIPT $DB2INST" \
        MonitorCommand="$DB2_MONITOR_SCRIPT $DB2INST" \
        MonitorCommandPeriod=$DB2_MON_CMD_PERIOD \
        MonitorCommandTimeout=$DB2_MON_CMD_TIMEOUT \
        NodeNameList="{'$HOST','$node'}" \
        StartCommandTimeout=$DB2_MON_START_TIMEOUT \
        StopCommandTimeout=$DB2_MON_STOP_TIMEOUT \
        UserName="root" \
        ResourceType=1
        
        mkrg -p $DB2_RG_PRIORITY $DB2_RG
        debug "Adding db2 resources to db2 rg"
    	add_to_rg $DB2_DB_RSRC_NAME IBM.Application $DB2_RG  $ADDRGMBR_ORDER
        
    fi
	
    # The db2 resource group should now exist.
    is_rg_created $DB2_RG
    if [ $? -eq 0 ]; then
    	fatal_error "The DB2 resource group was not created."     
    fi
    
}

crt_db2_dep () {
	
	debug "Create DB2 dependencies"
	
	# Need to take the db2 rg offline before we can put a relation on it.
    take_rg_offline_n_wait $DB2_RG
	
    # DB2 depends on the drbd storage being mounted                    
    make_relationship DependsOn IBM.Application:${DB2_DB_RSRC_NAME} \
    IBM.Application:${DB2MIRROR_RSRC_NAME} \
    ${DB2_DB_RSRC_NAME}-depon-${DB2MIRROR_RSRC_NAME}
	
#    start_n_wait_for_rg_online $DB2_RG
	
	
}


comment_fmc () {
    perl -pi -e "s/^fmc:2345/#fmc:2345/g" /etc/inittab
}

uncomment_fmc () {
    perl -pi -e "s/^#fmc:2345/fmc:2345/g" /etc/inittab
}

db2auto_off () {
    $DB2SQL_DIR/bin/db2iauto -off $DB2INST
}

db2auto_on () {
    $DB2SQL_DIR/bin/db2iauto -on $DB2INST
}

kill_db2fmd () {
    local fmdps=`ps auxww | grep db2fmd | grep $DB2INST | awk '{print $2}'`
    if [ -n "$fmdps" ]; then 
        local result=`kill $fmdps`
        debug "kill db2fmd: ps=$fmdps result=$result"
    else
        debug "No db2fmd process found."
    fi

}

