#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996 
#

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh

ensure_multiple_paths () {
	IFname=$1

	RSRC_LIST=/tmp/KGNMAS.$$

	(( NodeCount = 2 ))
	(( subnetCount = 0 ))
	prevCGname='__kgnmas__'

	export IFS='|'

	LC_ALL=C lsrsrc-api -D '|' -s IBM.NetworkInterface::::CommGroup::NodeNameList::Name::HeartbeatActive::OpState | sort -t \| -k1,1 -k 2,2 > $RSRC_LIST

	while read x
	do
        set $x
        CGname=$1
        NNL=$2
	Name=$3
        HBA=$4
        OpState=$5

        # echo CGname="\"$CGname\"" NNL="\"$NNL\""  Name="\"$Name\"" HBA="\"$HBA\""  OpState="\"$OpState\""

	if [[ $Name == "$IFname" ]]
	then
		# echo "skip matching IF $Name"
		continue
	fi

        if [[ $HBA != "1" || $OpState != "1" ]]
        then
                # echo "bad HBA or OpState"
                continue
        fi

        if [[ "$prevCGname" != "$CGname" ]]
        then
                prevCGname=$CGname
                prevNNL=$NNL
                (( count = 1 ))

        elif [[ $prevNNL != $NNL ]]
        then
                prevNNL=$NNL
                (( count = count + 1 ))

                if (( NodeCount == count ))
                then
                        (( subnetCount = subnetCount + 1 ))
                fi
        else
                # start a new search

                prevCGname='__kgnmas__'
        fi
	done < $RSRC_LIST
	rm -f $RSRC_LIST

	echo $((subnetCount))
	debug "ensure_multiple_paths: found $subnetCount subnets"
	if (( subnetCount >= 1 ))
	then
        return 0
	else
        return 1
	fi
}


change_address () {
	
	# Change the config file
	perl -pi -e "s/${OLD_ADDRESS}/${NEW_ADDRESS}/g" $SMHA_CFG_FILE
	
    # Change the drbd config file
    perl -pi -e "s/${OLD_ADDRESS}/${NEW_ADDRESS}/g" $DRBD_CONF
    
    # Change the twgnew properties file
    perl -pi -e "s/${OLD_ADDRESS}/${NEW_ADDRESS}/g" $TWGNEW_PROP_FILE
    
    # Change the /etc/hosts file
    perl -pi -e "s/^${OLD_ADDRESS}/${NEW_ADDRESS}/g" /etc/hosts
    debug "change_address: restarting resolver cache"
    /etc/init.d/nscd restart > /dev/null
    
    #sync_tsa_config -- only do this on the opposite node

}

change_hostname () {
	
	# Change the config file
	# What happens when you change from two nodes "hogdog", hogdog1" to "hogdog0", "hogdog1"????
	#    Handle each possibility separately, so it all works.
    perl -pi -e "s/=${OLD_HOSTNAME}$/=${NEW_HOSTNAME}/g;s/=${OLD_HOSTNAME},/=${NEW_HOSTNAME},/g;s/,${OLD_HOSTNAME}$/,${NEW_HOSTNAME}/g" $SMHA_CFG_FILE

    # Change the daemon config file	
    perl -pi -e "s/${OLD_HOSTNAME}$/${NEW_HOSTNAME}/g" $DM_CONF
    
    # Change the drbd config file
    perl -pi -e "s/on ${OLD_HOSTNAME}/on ${NEW_HOSTNAME}/g" $DRBD_CONF
    
    # Change the /etc/hosts file
    perl -pi -e "s/ ${OLD_HOSTNAME}$/ ${NEW_HOSTNAME}/g;s/ ${OLD_HOSTNAME} / ${NEW_HOSTNAME} /g;s/ ${OLD_HOSTNAME}\./ ${NEW_HOSTNAME}./g" /etc/hosts
    debug "restarting resolver cache in change_hostname"
    /etc/init.d/nscd restart
    #sync_tsa_config  -- only do this on the opposite node
}

update_etc_hosts () {
	
	#Handle three update conditions: address, host, both
	
	# Both (This probably will never happen since the chnetcfg cmd doesn't support it)
	if [ -n "$NEW_ADDRESS" -a -n "$NEW_HOSTNAME" ]; then
		# if we find the address in the file then we're all set, else update the file
		local findaddr=`grep $NEW_ADDRESS /etc/hosts`
		if [ -z "$findaddr" ]; then
			add_etc_hosts_entry $NEW_ADDRESS $NEW_HOSTNAME
		fi

    # New Address		
    elif [ -n "$NEW_ADDRESS" ]; then
        # if we find the address in the file then we're all set, else update the file
        local findaddr=`grep $NEW_ADDRESS /etc/hosts`
        if [ -z "$findaddr" ]; then
        	debug "Did not find new address in /etc/hosts"
        	##### Reconsidered...we shouldn't update the /etc/hosts if it wasn't already present.
        	# need to update the file
        	# get the existing node name
        	# get_node_value        	
            #add_etc_hosts_entry $NEW_ADDRESS $node
        else
            debug "Found new address in /etc/hosts already"
        fi

    
    # New Hostname
    elif [ -n "$NEW_HOSTNAME" ]; then
        local findhost=`grep -e " ${NEW_HOSTNAME}$" /etc/hosts`
        debug "Looked for ${NEW_HOSTNAME} in /etc/hosts, grep returned ${findhost} in stdout"
        if [ -z "$findhost" ]; then
        	# We didn't find an entry
        	debug "New hostname not found in /etc/hosts"
        	# Try pinging the new host 
        	ping_host $NEW_HOSTNAME
        	if [ $? -ne 0 ]; then
        		# ping didn't work, use the old address if we can ping it.
        		# What's actually passed in this case is possibly a set of addresses with comma separator
        		local addresses=`echo $OLD_ADDRESS | sed 's/,/ /g'`
        		local addr
        		for addr in $addresses
        		do	
	                ping_host $addr
	                if [ $? -eq 0 ]; then                                
	                    add_etc_hosts_entry $addr $NEW_HOSTNAME
	                    debug "Added entry to /etc/hosts: $addr $NEW_HOSTNAME"
	                    return
	                fi
        		done
        		 
        	fi
        fi
    
    
	fi
	
}

#sync_config () {
#	local RC=0
#	debug "SyncConfig processing"
#	sync_tsa_config
#	exit $RC
#}


chg_free_lock_on_signal () {
    rw_lock write free NameChange $PROGNAME

    trap - EXIT              # remove exit trap
    exit 1
}

chg_free_lock_on_exit () {
    rw_lock write free NameChange $PROGNAME
}

chg_get_write_lock () {
    local rc
    local timer=$1

    # set traps for common "termination" signals, and exit, in order to free the read lock
    #
    # N.B. due to a race, it is possible for the trap handler to be called before the read
    # lock is acquired. That is OK, since freeing a non-existent read lock is a no-op

    trap chg_free_lock_on_signal SIGHUP SIGINT SIGQUIT SIGABRT SIGTERM
    trap chg_free_lock_on_exit   EXIT

    rw_lock write get NameChange $PROGNAME $timer
    rc=$?

    if [ $rc -ne 0 ]; then
        trap - SIGHUP SIGINT SIGQUIT SIGABRT SIGTERM
        trap - EXIT
    fi

    return $rc
}

chg_free_write_lock () {
    rw_lock write free NameChange $PROGNAME

    trap - SIGHUP SIGINT SIGQUIT SIGABRT SIGTERM
    trap - EXIT
}

###################################
## Main code                     ##
###################################
RC=0
debug "smhachg called with $* "
local_change_only=

while getopts a:A:h:H:c:lSr flag;do
    case $flag in
        a) OLD_ADDRESS=$OPTARG ;;
        A) NEW_ADDRESS=$OPTARG ;;
        h) OLD_HOSTNAME=$OPTARG ;;
        H) NEW_HOSTNAME=$OPTARG ;;
		c) INTERFACE=$OPTARG ;;
        # Local change mode is used on the secondary node only.  It prevents the ping pong effect.
        l) local_change_only=1 ;;
        r) reset_management_IPs=1 ;;
        S) exit $RC ;;
        
    esac
done

if [ -n $OLD_ADDRESS ];then 
	# networkUtil is calling us with blah:blah/prefix for the -A parm
	if [[ "$OLD_ADDRESS" == */* ]]; then
		debug "old address contains a forward slash"
		# an ipv6 address with a netmask prefix on the end - remove the prefix
		OLD_ADDRESS=`echo $OLD_ADDRESS | cut -d/ -f1`
		debug "old address is : $OLD_ADDRESS"
	fi
fi

if [ -n "$INTERFACE" ]; then
	ensure_multiple_paths $INTERFACE > /dev/null
	RC=$?
	debug "smhachg -c $INTERFACE exit: RC=$RC"
	exit $RC
fi

if [ -n "$reset_management_IPs" ]; then
	refresh_managementIPs
	debug "smhachg exit; RC=$RC"
	exit $RC
fi

debug "Start hostname and address change processing"

if ! chg_get_write_lock 60; then
    debug "smhachg exit 1; cannot get write lock !!!"
    exit 1
fi

# We need to tell the other side about this first, BEFORE we start messing
# around with changing /etc/hosts 
# This is only a local change, don't send it to the other node
if [ -n "$local_change_only" ]; then
    debug "local_change_only, so do not send change request back to other node"
else
    # This change will be sent to the other node.
    
    # Set the sync conf flag so the tsa config gets sync'd on reboot
    #create_syncconf_flagfile
    
    # create the refreshManagementIP flag file. The maintenance task will 
    # wait for director to come online, and then reset management IPs as needed.
    create_refresh_managementIP_flagfile
    
    # if an old address is not supplied then send the list of old addresses
    # this would be the case when they are changing the hostname
    if [ -z "$OLD_ADDRESS" ]; then 
        OLD_ADDRESSES=`lsnetcfg -n -F ipaddr | sed 's/\"//g'`
        OLD_ADDRESSES="-a $OLD_ADDRESSES"
    fi
    run_init "hostnameaddress" $* $OLD_ADDRESSES
fi

# Always update the address first, then the hostname.
# This ensures that if we put entries in /etc/hosts the address is the new one.
if [ -n "$NEW_ADDRESS" ]; then	
    debug "Changing the address"             
    change_address
fi
if [ -n "$NEW_HOSTNAME" ]; then
    debug "Changing the hostname on the local node only"             
    change_hostname	
fi

if [ -n "$local_change_only" ]; then
    update_etc_hosts
fi

chg_free_write_lock

debug "End of hostname and address change processing"
debug "smhachg exit; RC=$RC"
exit $RC
