#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996 
#
PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1 
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh


debug "smhapause.sh called with $* "

action=
local_change_only=

while getopts p:r:l  flag;do
    case $flag in
        p) action=pause
           reason=${OPTARG} 
           ;;
        r) action=resume
     	   reason=${OPTARG} 
     	   ;;
        # Local change mode is used on the secondary node only.  It prevents the ping pong effect.
        l) local_change_only=1 ;;
    esac
done

rc=0 

is_passive
if [ $? -eq 0 ]; then
	# Active node

    if [ "$action" = "pause" -a "$reason" = "drbdNoStart" ]; then
		# send the same request to the passive
		debug "invoking Initializer with modifyReplication ${action} ${reason}"
		run_init "modifyReplication" ${action} ${reason}
		rc=$?
	fi
else
	# passive node.  We could be called by: 
	# - update script during a reboot of the passive, in which case we don't want to do anything
	# - HA daemon, in which case the -l flag was set, in which case we want to do it locally
	if [ -n "$local_change_only" ]; then
		# -l was passed, so process locally.
        if [ "$action" = "pause" -a "$reason" = "drbdNoStart" ]; then
			touch ${DRBD_NOSTART_FLAG}
		fi
	fi
fi

debug "smhapause.sh exiting with rc=$rc"

exit $rc
	
