#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

# 
# 09N6996
#

PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################

modify_service() {
	service=${1}
	status=${2}
	
case ${status} in 
on) 
	# turn on the service
	chkconfig_out=`/sbin/chkconfig --add ${service}`
	rc=$?
	debug "/sbin/chkconfig --add ${service}: output: ${chkconfig_out} : rc: $rc"
	# now start the service
	service_out=`/etc/init.d/${service} start`
	rc=$?
	debug "/etc/init.d/${service} start : output: ${service_out} : rc: $rc"
	;;
off)
	# turn off the service
	chkconfig_out=`/sbin/chkconfig --del ${service}`
	rc=$?
	debug "/sbin/chkconfig --del ${service}: output: ${chkconfig_out} : rc: $rc"
	# now stop the service
	service_out=`/etc/init.d/${service} stop`
	rc=$?
	debug "/etc/init.d/${service} stop : output: ${service_out} : rc: $rc"
	;;
esac
}

director_nonstop() {
	status=${1}
	
	case ${status} in
	enable)
		perl -pi -e "s/^restart=1/restart=5/g" /opt/ibm/director/lwi/runtime/nonstop/config/nonstop.properties
		;;
	disable)
		perl -pi -e "s/^restart=5/restart=1/g" /opt/ibm/director/lwi/runtime/nonstop/config/nonstop.properties
		;;
	esac
}

#####################################################################
# On the appliance, by default, the following services are installed as "on":
# cimserverd
# 

Action=${1}
position=${2}

debug "smhaservices.sh called with ${1} ${2}"

# when configuration is started, "conf" is invoked with "begin", then "end"
# when configuration is terminated, "unconf" is invoked with "begin", then "end"
case ${Action} in
conf)
	if [ "$position" == "begin" ]
	then
		modify_service cimlistenerd off
		modify_service cimserverd off
		modify_service tier1slpinst off
		modify_service paslpd off	
		modify_service ntp off
	fi
	RC=0
    ;;
unconf)
	if [ "$position" == "end" ]
	then
		modify_service paslpd on
		modify_service tier1slpinst on
		modify_service cimserverd on
		modify_service cimlistenerd on
		modify_service ntp on
		activate_dir_server_nonstop
	fi
	RC=0
    ;;
upgrade) 
	modify_service cimlistenerd off
	modify_service cimserverd off
	modify_service tier1slpinst off
	modify_service paslpd off	
	modify_service ntp off
	# only need the upgrade case for DHCP since that is already
	# hardcode in other scripts for configure/unconfigure
	modify_service dhcpd off
	#TODO: Decide whether to activate or deactivate _dir_server_nonstop on an upgrade
	RC=0
	;;
esac
debug "smhaservices.sh exiting."
exit $RC