# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# basic way we print out fstab lines
function echo(part, name, type, params)
{
    if (type != "swap") {
      print part "\t" prefix name "\t" type "\t" params
    } else {
      print part "\t" name "\t" type "\t" params
    }
}

# wrapper around echo to support labels.
function echolabel(part, device, name, type, params)
{
    # if label is not set, then don't print label
    if (label != "") {
      ("/opt/ibm/vmi/bin/convert_mnt2label " name " " device)|getline part
      echo("LABEL=" part, name, type, params)
    } else {
      echo(part, name, type, params)
    }
}


BEGIN  {
  base_count = 0;
  skip_count = 0;
  FS=":"

  split(v, args)
  d=args[1]
  dump=args[2]
  label=args[3]
  prefix=args[4]
  mode=args[5]
}

# Swap parition
/:swap/ {
  if (and(mode, 8) == 8) {
    echolabel(d $1, d, $2, "swap", "pri=42 0 0")
  }
  skip_count++
}

# Ignore extended
/:extended/ {
  skip_count++
}

# handle dump paritions
/\/dump/ {
  if (dump != "")
  {
    if (and(mode, 4) == 4) { 
      echolabel(dump "1", dump, $2, "ext3", "defaults 1 1")
    }
    skip_count++
  }
}

# Handle normal
/^[0-9]+/ {
  # print skip_count " " base_count
  if (skip_count == base_count)
    # if we don't override mount flags, then use the default
    if ($3 == "") {
      if (and(mode, 1) == 1) {
        echolabel(d $1, d, $2, "ext3", "defaults 1 1")
      }
    } else {
      if (and(mode, 1) == 1) {
        echolabel(d $1, d, $2, "ext3", $3)
      }
    }
  else
    base_count++
}

# These are bind mounts
/^\// && !/^\/dev/ {
  if (and(mode, 2) == 2)
    echo($1, $2, "bind", "defaults,bind 0 0")
}

END {
  if (and(mode, 16) == 16) {
    echo("proc", "/proc", "proc", "defaults 0 0")
    echo("devpts", "/dev/pts", "devpts", "defaults 0 0")
    echo("usbdevfs", "/proc/bus/usb", "usbdevfs", "defaults,noauto 0 0")
    echo("/dev/cdrom", "/media/cdrom", "auto", "rw,noauto,owner,user,exec 0 0")
    echo("/dev/fd0", "/media/floppy", "auto", "rw,sync,noauto,owner,user,exec 0 0")
  }
}
