#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

SMHA_DEBUG_TSA=ON

source /opt/ibm/director/ha/bin/include/smha_exploiter_common.sh

#############################################################

print_usage () {
    echo -e "\
smhaemctrl.sh { -e | -h | --help }\n\
     -e      enable HA resource emulation\n\
     -h      display this help\n\
     --help  display this help\n\
\n\
  OR\n\
\n\
smhaemctrl.sh rsrc_name OpState[=time_value] ...\n\
     where rsrc_name is one of\n\
                     $SMHA_CIM_RSRC_NAME_LISTENER\n\
                     $SMHA_CIM_RSRC_NAME_SERVER\n\
                     $SMHA_CIM_RSRC_NAME_TIER1SLP\n\
                     $SMHA_CIM_RSRC_NAME_PASLP\n\
                     $SMHA_CIM_RSRC_NAME_IBMSA\n\
                     $SMHA_DB2_DB_RSRC_NAME\n\
                     $SMHA_DIRECTOR_RSRC_NAME\n\
                     $SMHA_DRBDB_RSRC_NAME\n\
                     $SMHA_DRBDM_MIRROR0_RSRC_NAME\n\
                     $SMHA_DRBDM_UPDATESLIB_RSRC_NAME\n\
                     $SMHA_DRBDP_MIRROR0_RSRC_NAME\n\
                     $SMHA_DRBDP_UPDATESLIB_RSRC_NAME\n\
                     $SMHA_DRBDS_MIRROR0_RSRC_NAME\n\
                     $SMHA_DRBDS_UPDATESLIB_RSRC_NAME\n\
                     $SMHA_NTP_RSRC_NAME\n\
     where OpState is one of\n\
                     unknown\n\
                     online\n\
                     offline\n\
                     failed   (indicating failed offline)\n\
                     stuck    (indicating stuck online)\n\
                     ponline  (indicating pending online)\n\
                     poffline (indicating pending online)\n\
           If the OpState value is prepended with 'L', then the value\n\
           is locked, i.e. it will not be changed by a subsequent\n\
           start or stop command.\n\
     where time_value multiplied by the MonitorCommandPeriod for\n\
     the specified resources is the time in seconds to maintain\n\
     the OpState before setting the next OpState value, if any\n\
\n\
  OR\n\
\n\
smhaemctrl.sh { -c | -s | -q } rsrc_name\n\
     -c      create Condition/Response resources for a 'failover driver'\n\
             for the specified HA resource.\n\
     -s      start the Condition/Response for the specified HA resource.\n\
     -q      stop the Condition/Response for the specified HA resource.\n\
     where rsrc_name is one of those listed above\n\
\n\
Example:\n\
          smhaemctrl.sh db2 poffline=60 offline=120 ponline=30 online\n\
          smhaemctrl.sh ntp Loffline"
}

mk_condresp () {
    for NodeName in $(lsrsrc-api -s IBM.PeerNode::::Name)
    do
        mkcondition -r IBM.Application \
                    -e "OpState != OpState@P" \
                    -d "Monitor OpState of $SMHA_DIRECTOR_RSRC_NAME resources" \
                    -m p \
                    -g 2 \
                    -s "Name == '$SMHA_DIRECTOR_RSRC_NAME'" \
                    -p $NodeName \
                    smha_${RSRC_NAME}_opstate

        mkresponse -n process_director_opstate \
                    -e A \
                    -p $NodeName \
                    -s "/opt/ibm/director/ha/bin/test/smhaevent.sh $RSRC_NAME" \
                    smha_${RSRC_NAME}_opstate

        mkcondresp smha_${RSRC_NAME}_opstate:${NodeName} smha_${RSRC_NAME}_opstate
    done
}

start_condresp () {
    for NodeName in $(lsrsrc-api -s IBM.PeerNode::::Name)
    do
        startcondresp smha_${RSRC_NAME}_opstate:${NodeName}
    done
}

stop_condresp () {
    for NodeName in $(lsrsrc-api -s IBM.PeerNode::::Name)
    do
        stopcondresp smha_${RSRC_NAME}_opstate:${NodeName}
    done
}

# verify there is at least one argument

if [ $# -lt 1 ]
then
    print_usage
    exit 1
fi

Arg=$1      # first argument is assumed to be resource name
shift

# check for flag argument

CondOp=    # initialize to NULL string

if [ ${Arg:0:1} = "-" ]
then
    if [ $Arg = "-h" -o $Arg = "--help" ]
    then
        print_usage
        exit 0
    elif [ $Arg = "-e" ]
    then
        echo "/opt/ibm/director/ha/bin/test" > /opt/ibm/director/ha/config/smha_emulation
        exit 0
    elif [ $Arg = "-c" ]
    then
        CondOp=make       # make Condition/EventResponse/Association resources
        Arg=$1
    elif [ $Arg = "-s" ]
    then
        CondOp=start      # start Condition/Response
        Arg=$1
    elif [ $Arg = "-q" ]
    then
        CondOp=stop       # stop Condition/Response
        Arg=$1
    else
        echo -e "invalid option specified: $Arg\n"
        print_usage
        exit 1
    fi
fi

# check that emulation is enabled

if [ ! -s /opt/ibm/director/ha/config/smha_emulation ]
then
    echo -e "emulation is not enabled\n"
    print_usage
    exit 1
fi

RSRC_NAME=$Arg

case $RSRC_NAME in
    $SMHA_CIM_RSRC_NAME_LISTENER) ;;
    $SMHA_CIM_RSRC_NAME_SERVER) ;;
    $SMHA_CIM_RSRC_NAME_TIER1SLP) ;;
    $SMHA_CIM_RSRC_NAME_PASLP) ;;
    $SMHA_CIM_RSRC_NAME_IBMSA) ;;
    $SMHA_DB2_DB_RSRC_NAME) ;;
    $SMHA_DIRECTOR_RSRC_NAME) ;;
    $SMHA_DRBDB_RSRC_NAME) ;;
    $SMHA_DRBDM_MIRROR0_RSRC_NAME) ;;
    $SMHA_DRBDM_UPDATESLIB_RSRC_NAME) ;;
    $SMHA_DRBDP_MIRROR0_RSRC_NAME) ;;
    $SMHA_DRBDP_UPDATESLIB_RSRC_NAME) ;;
    $SMHA_DRBDS_MIRROR0_RSRC_NAME) ;;
    $SMHA_DRBDS_UPDATESLIB_RSRC_NAME) ;;
    $SMHA_NTP_RSRC_NAME) ;;
    *)  echo -e "invalid resource name $Arg\n"
        print_usage
        exit 1 ;;
esac

if [ -n "$CondOp" ]
then
    case $CondOp in
        make)  mk_condresp ;;
        start) start_condresp ;;
        stop)  stop_condresp ;;
    esac
    exit 0
fi

# get value of MonitorCommandPeriod for the local constituent resource

# limit search to local resources with CT_MANAGEMENT_SCOPE=1

MonPer=$(CT_MANAGEMENT_SCOPE=1 lsrsrc-api -s "IBM.Application::Name == '$RSRC_NAME' && ResourceType == 0::MonitorCommandPeriod" 2>&1)
if [ $? -ne 0 ]
then
    smha_fatal_error 9 "Cannot fetch MonitorCommandPeriod for resource $RSRC_NAME: $MonPer"
fi

if [ $# -lt 1 ]
then
    echo -e "At least one OpState argument must be specified\n"
    print_usage
    exit 1
fi

RSRC_DATA=/opt/ibm/director/ha/bin/test/data
if [ ! -d $RSRC_DATA ]
then
    mkdir $RSRC_DATA
    chmod 700 $RSRC_DATA
fi

while [ $# -gt 0 ]
do
    Arg=$1
    shift

    OpState=${Arg%=*}      # get OpState value
    WaitTime=${Arg#*=}     # get WaitTime value, if any
    if [ $OpState = $WaitTime ]
    then
        WaitTime=       # Indicate no wait time
    fi

    if [ ${OpState:0:1} = 'L' ]
    then
        OpState=${OpState:1}
        OpStateLock=" locked"
    else
        OpStateLock=
    fi

    case $OpState in
        unknown)  OPSTATE=$TSA_UNKNOWN ;;
        online)   OPSTATE=$TSA_ONLINE ;;
        offline)  OPSTATE=$TSA_OFFLINE ;;
        failed)   OPSTATE=$TSA_FAILED_OFFLINE ;;
        stuck)    OPSTATE=$TSA_STUCK_ONLINE ;;
        ponline)  OPSTATE=$TSA_PENDING_ONLINE ;;
        poffline) OPSTATE=$TSA_PENDING_OFFLINE ;;
        *)  echo -e "Invalid OpState argument: $OpState\n"
            print_usage
            exit 1 ;;
    esac

    smha_debug_tsa "set OPSTATE ${OPSTATE}${OpStateLock} in ${RSRC_DATA}/${RSRC_NAME}"
    echo "${OPSTATE}${OpStateLock}" > ${RSRC_DATA}/${RSRC_NAME}

    if [ -n "$WaitTime" ]
    then
        WaitTime=$(( WaitTime * MonPer ))
        smha_debug_tsa "sleeping for $WaitTime seconds"
        sleep $WaitTime
    fi
done


exit 0
